@@ -1,5 +1,5 @@
 # mail.pm - This module includes mail sending functions and does the smtp job.
-# RCS Identication ; $Revision: 6115 $ ; $Date: 2009-08-17 18:08:08 +0200 (lun 17 aoû 2009) $ 
+# RCS Identication ; $Revision: 6752 $ ; $Date: 2010-11-17 08:23:05 +0100 (mer 17 nov 2010) $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -42,7 +42,7 @@ our @EXPORT = qw(mail_file mail_message mail_forward set_send_spool);
 
 
 ## RCS identification.
-#my $id = '@(#)$Id: mail.pm 6115 2009-08-17 16:08:08Z olivier.salaun $';
+#my $id = '@(#)$Id: mail.pm 6752 2010-11-17 07:23:05Z david.verdin $';
 
 my $opensmtp = 0;
 my $fh = 'fh0000000000';	## File handle for the stream.
@@ -89,18 +89,24 @@ sub set_send_spool {
 # IN : -$filename(+) : tt2 filename (with .tt2) | ''
 #      -$rcpt(+) : SCALAR |ref(ARRAY) : SMTP "RCPT To:" field
 #      -$data(+) : used to parse tt2 file, ref(HASH) with keys :
-#        -return_path(+) : SMTP "MAIL From:" field if send by smtp, 
-#                          "X-Sympa-From:" field if send by spool
-#        -to : "To:" header field
-#        -lang : tt2 language if $filename
-#        -list :  ref(HASH) if $sign_mode = 'smime', keys are :
-#          -name
-#          -dir
-#        -from : "From:" field if not a full msg
-#        -subject : "Subject:" field if not a full msg
-#        -replyto : "Reply-to:" field if not a full msg
-#        -body  : body message if not $filename
-#        -headers : ref(HASH) with keys are headers mail
+#         -return_path(+) : SMTP "MAIL From:" field if send by smtp, 
+#                           "X-Sympa-From:" field if send by spool
+#         -to : "To:" header field
+#         -lang : tt2 language if $filename
+#         -list :  ref(HASH) if $sign_mode = 'smime', keys are :
+#            -name
+#            -dir
+#         -from : "From:" field if not a full msg
+#         -subject : "Subject:" field if not a full msg
+#         -replyto : "Reply-to:" field if not a full msg
+#         -body  : body message if not $filename
+#         -headers : ref(HASH) with keys are headers mail
+#         -dkim : a set of parameters for appying DKIM signature
+#            -d : d=tag
+#            -i : i=tag (optionnal)
+#            -selector : dkim dns selector
+#            -header_list : headers part of the signed infos
+#            -key : the RSA private key
 #      -$robot(+)
 #      -$sign_mode :'smime' | '' | undef
 #         
@@ -112,6 +118,7 @@ sub mail_file {
     my $header_possible = $data->{'header_possible'};
     my $sign_mode = $data->{'sign_mode'};
 
+
     &do_log('debug2', 'mail::mail_file(%s, %s, %s)', $filename, $rcpt, $sign_mode);
 
     my ($to,$message);
@@ -130,7 +137,9 @@ sub mail_file {
     }
 
     ## Charset for encoding
+    &Language::PushLang($data->{'lang'}) if defined $data->{'lang'};
     $data->{'charset'} ||= &Language::GetCharset();
+    &Language::PopLang() if defined $data->{'lang'};
 
     ## TT2 file parsing 
     if ($filename =~ /\.tt2$/) {
@@ -233,11 +242,14 @@ sub mail_file {
     ## Determine what value the Auto-Submitted header field should take
     ## See http://www.tools.ietf.org/html/draft-palme-autosub-01
     ## the header filed can have one of the following values : auto-generated, auto-replied, auto-forwarded
-    #unless ($header_ok{'auto_submitted'}) {
-    #  ## Default value is 'auto-generated'
-    #  my $header_value = $data->{'auto_submitted'} || 'auto-generated';
-    #  $headers .= "Auto-Submitted: $header_value\n"; 
-    #}
+    ## The header should not be set when wwsympa sends a command/mail to sympa.pl through its spool
+    unless ($data->{'not_auto_submitted'} ||
+	    $header_ok{'auto_submitted'}) {
+      ## Default value is 'auto-generated'
+      my $header_value = $data->{'auto_submitted'} || 'auto-generated';
+      $headers .= "Auto-Submitted: $header_value\n"; 
+    }
+
     unless ($existing_headers) {
 	$headers .= "\n";
    }
@@ -279,7 +291,12 @@ sub mail_file {
 			     'robot' => $robot,
 			     'listname' => $listname,
 			     'priority' => &Conf::get_robot_conf($robot,'sympa_priority'),
-			     'sign_mode' => $sign_mode)) {
+			     'sign_mode' => $sign_mode,
+			     'use_bulk' => $data->{'use_bulk'},
+			     'dkim' => $data->{'dkim'},
+			     )
+	    )
+    {
 	return undef;
     }
    return 1;
@@ -299,8 +316,14 @@ sub mail_file {
 #       
 ####################################################
 sub mail_message {
-    my($message, $list, $verp, @rcpt) = @_;
-   
+
+    my %params = @_;
+    my $message =  $params{'message'};
+    my $list =  $params{'list'};
+    my $verp = $params{'verp'};
+    my @rcpt =  @{$params{'rcpt'}};
+    my $dkim  =  $params{'dkim_parameters'};
+    my $tag_as_last = $params{'tag_as_last'};
 
     my $host = $list->{'admin'}{'host'};
     my $robot = $list->{'domain'};
@@ -308,7 +331,7 @@ sub mail_message {
     # normal return_path (ie used if verp is not enabled)
     my $from = $list->{'name'}.&Conf::get_robot_conf($robot, 'return_path_suffix').'@'.$host;
 
-    do_log('debug', 'mail::mail_message(from: %s, , file:%s, %s, verp->%s, %d rcpt)', $from, $message->{'filename'}, $message->{'smime_crypted'}, $verp->{'enable'}, $#rcpt+1);
+    do_log('debug', 'mail::mail_message(from: %s, , file:%s, %s, verp->%s, %d rcpt, last: %s)', $from, $message->{'filename'}, $message->{'smime_crypted'}, $verp, $#rcpt+1, $tag_as_last);
     
     my($i, $j, $nrcpt, $size); 
     my $numsmtp = 0;
@@ -399,7 +422,10 @@ sub mail_message {
 				'robot' => $robot,
 				'encrypt' => $message->{'smime_crypted'},
 				'use_bulk' => 1,
-				'verp' => $verp->{'enable'} ));
+				'verp' => $verp,
+				'dkim' => $dkim,
+				'merge' => $list->{'admin'}{'merge_feature'},
+				'tag_as_last' => $tag_as_last));
 }
 
 
@@ -434,10 +460,11 @@ sub mail_forward {
 	&do_log('err',"Unknown message variable type: '%s'. can't process message forwarding.",ref($msg));
 	return undef;
     }
+
     ## Add an Auto-Submitted header field according to  http://www.tools.ietf.org/html/draft-palme-autosub-01
-    #if (defined $message and $message->head) {
-    #	$message->head->add('Auto-Submitted', 'auto-forwarded');
-    #}
+    if (defined $message && $message->head) {
+      $message->head->add('Auto-Submitted', 'auto-forwarded');
+    }
     
     unless (defined &sending('msg' => $messageasstring, 
 			     'rcpt' => $rcpt,
@@ -499,6 +526,7 @@ sub reaper {
 #     $robot(+) : robot 
 #     $encrypt : 'smime_crypted' | undef
 #     $verp : 1| undef  
+#     $merge : 1| undef  
 #     $use_bulk : if defined,  send message using bulk
 #     
 # OUT : 1 - call to sending
@@ -515,9 +543,12 @@ sub sendto {
     my $priority =  $params{'priority'}; 
     my $encrypt = $params{'encrypt'};
     my $verp = $params{'verp'};
+    my $merge = $params{'merge'};
+    my $dkim = $params{'dkim'};
     my $use_bulk = $params{'use_bulk'};
-    
-    do_log('debug', 'mail::sendto(from : %s,listname: %s, encrypt : %s, verp : %s, priority = %s', $from, $listname, $encrypt, $verp, $priority);
+    my $tag_as_last = $params{'tag_as_last'};
+
+    do_log('debug', 'mail::sendto(from : %s,listname: %s, encrypt : %s, verp : %s, priority = %s, last: %s', $from, $listname, $encrypt, $verp, $priority, $tag_as_last);
     
     my $delivery_date =  $params{'delivery_date'};
     $delivery_date = time() unless $delivery_date; # if not specified, delivery tile is right now (used for sympa messages etc)
@@ -526,6 +557,7 @@ sub sendto {
 
     if ($encrypt eq 'smime_crypted') {
         # encrypt message for each rcpt and send the message
+	# this MUST be moved to the bulk mailer. This way, merge will be applied after the SMIME encryption is applied ! This is a bug !
 	foreach my $unique_rcpt (@{$rcpt}) {
 	    my $email = lc(@{$unique_rcpt}[0]);
 	    if (($email !~ /@/) || ($#{@$unique_rcpt} != 0)) {
@@ -542,8 +574,10 @@ sub sendto {
 			 'robot' => $robot,
 			 'priority' => $priority,
 			 'delivery_date' =>  $delivery_date,
-			 'use_bulk' => $use_bulk );
-	    }    
+			 'use_bulk' => $use_bulk,
+			 'tag_as_last' => $tag_as_last);
+	    }
+	    $tag_as_last = 0;
 	}
     }else{
 	$msg = $msg_header->as_string . "\n" . $msg_body;   
@@ -557,7 +591,10 @@ sub sendto {
 				  'priority' => $priority,
 				  'delivery_date' =>  $delivery_date,
 				  'verp' => $verp,
-				  'use_bulk' => $use_bulk);
+				  'merge' => $merge,
+				  'use_bulk' => $use_bulk,
+				  'dkim' => $dkim,
+				  'tag_as_last' => $tag_as_last);
 	    return $result;
 	}else{
 	    return undef;
@@ -582,6 +619,7 @@ sub sendto {
 #      -$listname : listname | ''
 #      -$sign_mode(+) : 'smime' | 'none' for signing
 #      -$verp 
+#      -dkim : a hash for dkim parameters
 #
 # OUT : 1 - call to smtpto (sendmail) | 0 - push in spool
 #           | undef
@@ -590,6 +628,7 @@ sub sendto {
 sub sending {
     my %params = @_;
     my $msg = $params{'msg'};
+    my $msg_id;
     my $rcpt = $params{'rcpt'};
     my $from = $params{'from'};
     my $robot = $params{'robot'};
@@ -601,8 +640,10 @@ sub sending {
     my $delivery_date = $params{'delivery_date'};
     $delivery_date = time() unless ($delivery_date); 
     my $verp  =  $params{'verp'};
+    my $merge  =  $params{'merge'};
     my $use_bulk = $params{'use_bulk'};
-
+    my $dkim = $params{'dkim'};
+    my $tag_as_last = $params{'tag_as_last'};
     my $sympa_file;
     my $fh;
     my $signed_msg; # if signing
@@ -636,29 +677,44 @@ sub sending {
     #	$messageasstring = $signed_msg->{'msg_as_string'};
     #    }
     if (ref($msg) eq "MIME::Entity") {
-	$messageasstring = $msg->as_string;    
+	$messageasstring = $msg->as_string;
+	my $head = $msg->head;
+	$msg_id = $head->get('Message-ID');
     }else {
 	$messageasstring = $msg;
+	if ($messageasstring =~ /Message-ID:\s*(\<.*\>)\s*\n/) {
+	    $msg_id = $1;
+	}
     }
     my $verpfeature = ($verp eq 'on');
+    my $mergefeature = ($merge eq 'on');
 
     if ($use_bulk){ # in that case use bulk tables to prepare message distribution 
-      my $bulk_code = &Bulk::store('msg' => $messageasstring,
-				   'rcpts' => $rcpt,
-				   'from' => $from,
-				   'robot' => $robot,
-				   'listname' => $listname,
-				   'priority_message' => $priority_message,
-				   'priority_packet' => $priority_packet,
-				   'delivery_date' => $delivery_date,
-				   'verp' => $verpfeature);
-      unless (defined $bulk_code) {
-	&do_log('err', 'Failed to store message for list %s', $listname);
-	&List::send_notify_to_listmaster('bulk_error',  $robot, {'listname' => $listname});
-	return undef;
-      }
 
-    }elsif(defined $send_spool) { # in context wwsympa.fcgi do note send message to reciepients but copy it to standard spool 
+	my $bulk_code = &Bulk::store('msg' => $messageasstring,
+				     'msg_id' => $msg_id,
+				     'rcpts' => $rcpt,
+				     'from' => $from,
+				     'robot' => $robot,
+				     'listname' => $listname,
+				     'priority_message' => $priority_message,
+				     'priority_packet' => $priority_packet,
+				     'delivery_date' => $delivery_date,
+				     'verp' => $verpfeature,
+				     'merge' => $mergefeature,
+				     'dkim' => $dkim,
+				     'tag_as_last' => $tag_as_last,
+				     );
+
+	unless (defined $bulk_code) {
+	    &do_log('err', 'Failed to store message for list %s', $listname);
+	    &List::send_notify_to_listmaster('bulk_error',  $robot, {'listname' => $listname});
+	    return undef;
+	}
+	
+    }elsif(defined $send_spool) { # in context wwsympa.fcgi do not send message to reciepients but copy it to standard spool 
+	do_log('debug',"NOT USING BULK");
+
 	$sympa_email = &Conf::get_robot_conf($robot, 'sympa');	
 	$sympa_file = "$send_spool/T.$sympa_email.".time.'.'.int(rand(10000));
 	unless (open TMP, ">$sympa_file") {
@@ -688,6 +744,7 @@ sub sending {
 	    return undef;
 	}
     }else{ # send it now
+	do_log('debug',"NOT USING BULK");
 	*SMTP = &smtpto($from, $rcpt, $robot);	
 	print SMTP $messageasstring;	
 	unless (close SMTP) {
@@ -969,10 +1026,17 @@ sub fix_part($$$$) {
 
 	my $head = $part->head;
 	my $body = $bodyh->as_string;
+	my $wrap = $body;
+	if ($head->get('X-Sympa-NoWrap')) { # Need not wrapping
+	    $head->delete('X-Sympa-NoWrap');
+	} elsif ($eff_type eq 'text/plain' and
+		 lc($head->mime_attr('Content-type.Format')||'') ne 'flowed') {
+	    $wrap = &tools::wrap_text($body);
+	}
 	my $charset = $head->mime_attr("Content-Type.Charset") || $defcharset;
 
 	my ($newbody, $newcharset, $newenc) = 
-	    MIME::Charset::body_encode(Encode::decode('utf8', $body), $charset,
+	    MIME::Charset::body_encode(Encode::decode('utf8', $wrap), $charset,
 				       Replacement => 'FALLBACK');
 	if ($newenc eq $enc and $newcharset eq $charset and
 	    $newbody eq $body) {
